import 'package:firebase_core/firebase_core.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:global_configuration/global_configuration.dart';
import 'package:mini_app_likewallet/home/index.dart';
import 'package:mini_app_likewallet/otp_page/otp_page.dart';
import 'package:mini_app_likewallet/provider/home_provider/check_tx_provider.dart';
import 'package:mini_app_likewallet/provider/home_provider/counter.dart';
import 'package:mini_app_likewallet/provider/home_provider/home_provider.dart';
import 'package:mini_app_likewallet/provider/home_provider/index_provider/index_provider.dart';
import 'package:mini_app_likewallet/provider/home_provider/loading_provider/loading_provider.dart';
import 'package:mini_app_likewallet/provider/home_provider/lock_all_provider/lock_all_provider.dart';
import 'package:mini_app_likewallet/provider/home_provider/otp_provider/otp_provider.dart';
import 'package:mini_app_likewallet/provider/home_provider/receive_provider/receive_provider.dart';
import 'package:mini_app_likewallet/provider/home_provider/send_provider/send_provider.dart';
import 'package:mini_app_likewallet/receive/receive.dart';
import 'package:mini_app_likewallet/send/send.dart';
import 'package:mini_app_likewallet/send/send_confirm.dart';
import 'package:mini_app_likewallet/send/send_receipt.dart';
import 'package:mini_app_likewallet/theme/widget.dart';
import 'package:provider/provider.dart';

void runMain() async {
  print('wft');
  await Firebase.initializeApp();
  WidgetsFlutterBinding.ensureInitialized();
  try {
    await GlobalConfiguration()
        .loadFromUrl("https://new.likepoint.io/configAPI");
  } catch (e) {
    // something went wrong while fetching the config from the url ... do something
  }
}

class FirstPage extends StatelessWidget {
  FirstPage({this.phone});
  final String phone;
  @override
  Widget build(BuildContext context) {
    runMain();
    return MultiProvider(
      providers: [
        ChangeNotifierProvider.value(value: Counter()),
        ChangeNotifierProvider.value(value: HomeProvider()),
        ChangeNotifierProvider.value(value: SendProvider()),
        ChangeNotifierProvider.value(value: LockAllProvider()),
        ChangeNotifierProvider.value(value: OTPProvider()),
        ChangeNotifierProvider.value(value: LoadingProvider()),
        ChangeNotifierProvider.value(value: IndexProvider()),
        ChangeNotifierProvider.value(value: ReceiveProvider()),
        ChangeNotifierProvider.value(value: CheckTXProvider()),
      ],
      child: MaterialApp(
        localizationsDelegates: [
          // GlobalMaterialLocalizations.delegate,
          // GlobalWidgetsLocalizations.delegate,
          // GlobalCupertinoLocalizations.delegate
        ],
        theme: ThemeData(visualDensity: VisualDensity.adaptivePlatformDensity),
        home: MyHomePage(phone: phone),
      ),
    );
  }
}

enum providerSMS { Firebase, Twilio, Nexmo, Email }

class MyHomePage extends StatefulWidget {
  MyHomePage({this.phone});
  final String phone;
  @override
  _MyHomePage createState() => _MyHomePage();
}

class _MyHomePage extends State<MyHomePage> {
  _MyHomePage({this.phone});
  final String phone;
  HomeProvider _homeProvider;
  OTPProvider _otpProvider;
  LoadingProvider _loadingProvider;
  IndexProvider _indexProvider;
  providerSMS roundSMS = providerSMS.Twilio;
  @override
  void initState() {
    // TODO: implement initState
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    _homeProvider = Provider.of<HomeProvider>(context);
    _otpProvider = Provider.of<OTPProvider>(context);
    _loadingProvider = Provider.of<LoadingProvider>(context);
    _indexProvider = Provider.of<IndexProvider>(context);

    ScreenUtil.init(context,
        designSize: Size(1080, 2340), allowFontScaling: false);
    return Scaffold(
      backgroundColor: Colors.blueAccent,
      body: SingleChildScrollView(
        child: Container(
          height: MediaQuery.of(context).size.height,
          child: Stack(
            children: <Widget>[
              _homeProvider.getLikeWallet ? IndexScreen() : Container(),
              _homeProvider.getSend ? SendScreen() : Container(),
              _homeProvider.getSendConfirm ? SendConfirmScreen() : Container(),
              _homeProvider.getSendReceipt ? SendReceiptScreen() : Container(),
              _homeProvider.getReceive ? ReceiveScreen() : Container(),
              _homeProvider.getOTP ? OTPPageScreen() : Container(),
              FlatButton(
                  onPressed: () async {
                    bool login = await _otpProvider.checkLogin();
                    _loadingProvider.setStatus(status: true);
                    if (login) {
                      _homeProvider.setMenu(status: true);
                      await _indexProvider.loadBalance();
                      _loadingProvider.setStatus(status: false);
                    } else {
                      _homeProvider.setOTP(status: true);
                      await _otpProvider.setNumberPhone(phone: widget.phone);
                      final res = await _otpProvider.checkExistUser(
                          phoneNumber: _otpProvider.getNumberPhone);
                      if (res) {
                        // _loadingProvider.setStatus(status: false);
                        final res = await _otpProvider.sendCodeToPhoneNumber(
                            phoneNumber: _otpProvider.getNumberPhone);
                        if (res) {
                          _loadingProvider.setStatus(status: false);
                        } else {
                          _homeProvider.setOTP(status: false);
                          WidgetTheme.showShortToastRed('เกิดข้อผิดพลาด');
                        }
                      } else {
                        WidgetTheme.showShortToastRed('ไม่พบเบอร์ในระบบ');
                      }
                    }
                  },
                  child: Text('กด'))
            ],
          ),
        ),
      ),
    );
  }
}
