import 'dart:convert';

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:dio/dio.dart';
import 'package:http/http.dart' as http;
import 'package:flutter/material.dart';
import 'package:mini_app_likewallet/config/config.dart';
import 'package:mini_app_likewallet/library/auth.dart';
import 'package:mini_app_likewallet/library/configuration_service.dart';
import 'package:mini_app_likewallet/library/ethcontract.dart';
import 'package:mini_app_likewallet/middleware/check_auth.dart';
import 'package:mini_app_likewallet/middleware/get_mnemonic.dart';
import 'package:permission_handler/permission_handler.dart';

class SendProvider with ChangeNotifier {
  List<String> sendAmount = [];
  TextEditingController noteController = TextEditingController();
  String nameTo = '';
  String phone = '';
  String addressTo = '';
  String nameFOMR = '';
  bool loading = false;
  bool tapAddNote = false;
  bool confirm = false;
  double rateCurrency = 0.0;
  SendProvider();

  List<String> get getAmount => sendAmount;
  bool get getTapAddNote => tapAddNote;
  bool get getConfirm => confirm;
  String get getNameTo => nameTo;
  String get getNameFOMR => nameFOMR;
  String get getPhone => phone;
  bool get getLoading => loading;
  String get getAddressTo => addressTo;
  double get getRateCurrency => rateCurrency;

  BaseAuth auth;
  Dio dio = new Dio();

  setLoading({bool status}) {
    loading = status;
    notifyListeners();
  }

  changeAmount({var value}) {
    sendAmount += value;
    notifyListeners();
  }

  deleteAmount() {
    if (sendAmount.length >= 1) {
      sendAmount.removeLast();
    }
    notifyListeners();
  }

  setTextAmount({var number}) {
    sendAmount.add(number.toString());
    notifyListeners();
  }

  changeAddNote({bool status}) {
    tapAddNote = status;
    notifyListeners();
  }

  changeConfirm({bool status}) {
    confirm = status;
    notifyListeners();
  }

  void getNameTO({String address}) async {
    auth = Auth();
    addressTo = address;
    final _token = await auth.getTokenFirebase();
    try {
      Response response = await dio.post(Env.apiUrl + "/searchNameNew",
          data: {"term": address, "_token": _token});
      if (response.data["statusCode"] == 200) {
        print(response.data["result"]['name']);
        print(response.data["result"]);
        nameTo = response.data["result"]['name'];
        phone = response.data["result"]['phoneNumber'];
        loading = false;
        notifyListeners();
      }
    } catch (e) {
      print(e);
    }
  }

  void getNameFROM({String address}) async {
    auth = Auth();
    final _token = await auth.getTokenFirebase();
    try {
      Response response = await dio.post(Env.apiUrl + "/searchNameNew",
          data: {"term": address, "_token": _token});
      if (response.data["statusCode"] == 200) {
        print(response.data["result"]['name']);
        print(response.data["result"]);
        nameFOMR = response.data["result"]['name'];
        notifyListeners();
      }
    } catch (e) {
      print(e);
    }
  }

  Future<bool> statusPermission() async {
    var status = await Permission.photos.status;
    final permissionStatus = await Permission.photos.request();
    if (permissionStatus == PermissionStatus.granted) {
      return true;
    } else if (status.isDenied ||
        status.isUndetermined ||
        status.isPermanentlyDenied) {
      return false;
    }
  }

  getRateCurrencySymbol({String symbol}) {
    FirebaseFirestore.instance
        .collection('exchangeFiat')
        .doc(symbol == 'none' ? "THB-LIKE" : "THB-" + symbol)
        .get()
        .then((DocumentSnapshot ds) {
      print(ds.data()["rate"]);
      if (ds.data()["to"] == 'LIKE') {
        rateCurrency = ds.data()["rate"];
      } else {
        rateCurrency = 1 / ds.data()["rate"];
        print(rateCurrency);
      }
    });
    notifyListeners();
  }

  Future<String> signTransaction({
    IConfigurationService configETH,
    EthContract eth,
    CurCheckAuth login,
    GetMnemonic seed,
    String addressTo,
    String amount,
    String contract,
    String callFunction,
    String abi,
  }) async {
    double rate = 100;
    String tx = '';
    num avaiBalance = await eth.getBalance(address: configETH.getAddress());
    if ((avaiBalance >= double.parse(amount) * rateCurrency * rate)) {
      //เช็ต login
      bool loginStatus = await login.checkLogin();
      if (loginStatus) {
        //เช็ต Seed กระเป๋า
        String value = await seed.getMnemonic();
        if (value.isNotEmpty) {
          // final mnemonic = value.split(":")[0];
          final pkEth = value.split(":")[1];

//        eth.sendTransaction(pk: pketh, to: nameTO.trim(), value: (double.parse(amount)*rateCurrency*rate).toString()).then((tx) {
          if (contract == 'no') {
            final resultTX = await eth.transferMessage(
                pk: pkEth,
                to: addressTo.trim(),
                value: (double.parse(amount) * rateCurrency * rate).toString(),
                message: noteController.text.toString());

            tx = resultTX;
            print('tx : $tx');
          } else if (contract.length == 42 &&
              callFunction != 'no' &&
              abi != 'no') {
            final resultTX = await eth.dynamicSendContract(
                pk: pkEth,
                to: addressTo.trim(),
                value: (double.parse(amount) * rateCurrency * rate).toString(),
                message: noteController.text.toString(),
                callFunction: callFunction,
                abi: abi,
                contractAddress: contract);
            tx = resultTX;
            print('tx : $tx');
          }
        }
      }
    } else {
      print('คุณมี LIKE ไม่เพียงพอ');
      // showShortToast(AppLocalizations.of(context).translate('not_enough') +
      //     AppLocalizations.of(context).translate('not_enough_tail') +
      //     " " +
      //     (avaiBalance).toString() +
      //     "LIKE");
    }
    return tx;
  }
}
