import 'dart:ui';

import 'package:flutter/material.dart';

import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:mini_app_likewallet/device_utils.dart';
import 'package:mini_app_likewallet/home/home_component/home_component.dart';
import 'package:mini_app_likewallet/library/address_service.dart';
import 'package:mini_app_likewallet/library/configuration_service.dart';
import 'package:mini_app_likewallet/library/ethcontract.dart';
import 'package:mini_app_likewallet/middleware/check_auth.dart';
import 'package:mini_app_likewallet/middleware/get_mnemonic.dart';
import 'package:mini_app_likewallet/provider/home_provider/check_tx_provider.dart';
import 'package:mini_app_likewallet/provider/home_provider/home_provider.dart';
import 'package:mini_app_likewallet/provider/home_provider/send_provider/send_provider.dart';
import 'package:mini_app_likewallet/send/component/send_confirm_component.dart';
import 'package:mini_app_likewallet/theme/custom_loading.dart';
import 'package:mini_app_likewallet/theme/widget.dart';
import 'package:modal_progress_hud/modal_progress_hud.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

class SendConfirmScreen extends StatefulWidget {
  @override
  _SendConfirmScreen createState() => _SendConfirmScreen();
}

class _SendConfirmScreen extends State<SendConfirmScreen> {
  SendProvider _sendProvider;
  CheckTXProvider _checkTXProvider;
  HomeProvider _getLikeWallet;
  IAddressService addressService;
  IConfigurationService configETH;
  CurCheckAuth login;
  BaseETH eth;
  GetMnemonic seed;
  String mnemonic;
  double balanceLIKE = 0.00;

  @override
  void didChangeDependencies() async {
    super.didChangeDependencies();
    _getLikeWallet = Provider.of<HomeProvider>(context);
    _sendProvider = Provider.of<SendProvider>(context, listen: true);
    _checkTXProvider = Provider.of<CheckTXProvider>(context, listen: true);
    if (_sendProvider.getNameTo != '' && _sendProvider.getAmount != null) {
      _sendProvider.confirm = true;
    }
  }

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    login = new CurCheckAuth();
    eth = new EthContract();
    seed = new MnemonicRetrieve();
    setInitFirst();
  }

  setInitFirst() async {
    SharedPreferences pref = await SharedPreferences.getInstance();
    configETH = new ConfigurationService(pref);
    addressService = new AddressService(configETH);
    mnemonic = await configETH.getMnemonic();
    if (mnemonic == 'no') {
      print('ดีดไป Login');
    }
    // fiat = pref.getString('currency') ?? 'none';
    // print('fiat : ' + fiat);
    String getPK = addressService.getPrivateKey(mnemonic);
    configETH.setPrivateKey(getPK);
    addressService.getPublicAddress(getPK).then((address) {
      print(address.toString());
      configETH.setAddress(address.toString());
      eth.getBalance(address: address.hex).then((balance) {
        balanceLIKE = balance;
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    ScreenUtil.init(context,
        designSize: Size(1080, 2340), allowFontScaling: false);
    return ModalProgressHUD(
      opacity: 0.1,
      inAsyncCall: _sendProvider.getLoading,
      progressIndicator: CustomLoading(),
      child: Scaffold(
//      backgroundColor: Colors.black.withOpacity(0.1),
        body: GestureDetector(
          onTap: () {
            DeviceUtils.hideKeyboard(context);
          },
          child: Container(
            color: Colors.transparent,
            height: 1491.h,
            child: Stack(
              children: [
                AnimatedPositioned(
                    duration: Duration(milliseconds: 150),
                    top: _sendProvider.getConfirm ? 729.h : 0.h,
                    child: BackdropFilter(
                        filter: ImageFilter.blur(sigmaX: 1, sigmaY: 1),
                        child: sendConfirmBackground())),
                Container(
                  color: Colors.transparent,
                  height: 1064.1.h,
                  child: Stack(children: [
                    Positioned(
                        top: -0.h,
                        child: SendConfirmComponent.sendBackground()),
                    Column(children: [
                      SizedBox(height: 140.h),
                      Container(
                          height: 101.56.h,
                          margin: EdgeInsets.only(left: 75.w),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: [
                              HomeComponent.nameWallet(),
                              HomeComponent.openLikeWallet()
                            ],
                          )),
                      SizedBox(height: 139.9.h),
                      SendConfirmComponent.sendTotal(
                          amount: _sendProvider.getAmount.join()),
                      SizedBox(height: 89.h),
                      InkWell(
                          onTap: () {
                            // _sendProvider.changeConfirm(status: true);
                          },
                          child: SendConfirmComponent.sendTo(context: context)),
                      SizedBox(height: 95.h),
                      InkWell(
                          onTap: () =>
                              _sendProvider.changeAddNote(status: true),
                          child: _sendProvider.getTapAddNote
                              ? SendConfirmComponent.note(
                                  noteController: _sendProvider.noteController)
                              : SendConfirmComponent.addNote()),
                      Expanded(child: Container()),
                      InkWell(
                          onTap: () {
                            _getLikeWallet.setSendConfirm(status: false);
                            _getLikeWallet.setSend(status: true);
                            _sendProvider.sendAmount = [];
                            _sendProvider.nameTo = '';
                            _sendProvider.phone = '';
                          },
                          child: SendConfirmComponent.backButton()),
                      SizedBox(height: 90.1.h),
                    ]),
                  ]),
                )
              ],
            ),
          ),
        ),
      ),
    );
  }

  Widget sendConfirmBackground() => Container(
        decoration: new BoxDecoration(
            color: Colors.white.withOpacity(0.8),
            borderRadius: new BorderRadius.all(Radius.circular(40.0))),
        height: 762.h,
        width: 1080.w,
        child: Column(
          children: [
            SizedBox(height: 381.h),
            InkWell(
                onTap: () async {
                  _sendProvider.setLoading(status: true);
                  SharedPreferences pref =
                      await SharedPreferences.getInstance();
                  _sendProvider.getNameFROM(
                      address: pref.getString('addressETH'));
                  // print(double.parse(
                  //     _sendProvider.sendAmount.join().replaceAll(',', '')));
                  // print(balanceLIKE);

                  if (double.parse(_sendProvider.sendAmount
                          .join()
                          .replaceAll(',', '')) <=
                      balanceLIKE) {
                    await _sendProvider.getRateCurrencySymbol(symbol: 'none');
                    final tx = await _sendProvider.signTransaction(
                      configETH: configETH,
                      amount:
                          _sendProvider.sendAmount.join().replaceAll(',', ''),
                      eth: eth,
                      login: login,
                      seed: seed,
                      abi: 'no',
                      callFunction: 'no',
                      contract: 'no',
                      addressTo: _sendProvider.getAddressTo,
                    );
                    if (tx.isNotEmpty) {
                      final status =
                          await _checkTXProvider.checkStatusTX(tx: tx);
                      if (status) {
                        _sendProvider.setLoading(status: false);
                        _getLikeWallet.setSendReceipt(status: true);
                      }
                    } else {
                      print('tx ไม่มา');
                    }
                  } else {
                    print('ยอดเงินไม่พอ');
                  }
                  // _getLikeWallet.setSendReceipt(status: true);
                  // print(_getLikeWallet.getSendReceipt);
                },
                child: SendConfirmComponent.buttonConfirm()),
            SizedBox(height: 28.h),
            InkWell(
                onTap: () {
                  _getLikeWallet.setSendConfirm(status: false);
                  _getLikeWallet.setSend(status: true);
                  _sendProvider.sendAmount = [];
                  _sendProvider.nameTo = '';
                  _sendProvider.phone = '';
                },
                child: SendConfirmComponent.buttonCancel()),
          ],
        ),
      );
}
